import { environment } from "#environment/config";
import { cassandra } from "#lib/database";
import { mailerService } from "#lib/mailer";
import { pika } from "#lib/utils";
import {
	isValidDisplayName,
	isValidEmail,
	isValidPassword,
	isValidUsername,
} from "#lib/validation";

import { echo } from "@atums/echo";
import type {
	ExtendedRequest,
	RegisterRequest,
	RegisterResponse,
	RouteDef,
} from "#types/server";

const routeDef: RouteDef = {
	method: "POST",
	accepts: "application/json",
	returns: "application/json",
	needsBody: "json",
};

async function handler(
	_request: ExtendedRequest,
	requestBody: unknown,
): Promise<Response> {
	try {
		const { username, displayName, email, password } =
			requestBody as RegisterRequest;

		if (!username || !email || !password) {
			const response: RegisterResponse = {
				code: 400,
				success: false,
				error: "Missing required fields: username, email, password",
			};
			return Response.json(response, { status: 400 });
		}

		const usernameValidation = isValidUsername(username);
		if (!usernameValidation.valid || !usernameValidation.username) {
			const response: RegisterResponse = {
				code: 400,
				success: false,
				error: usernameValidation.error || "Invalid username",
			};
			return Response.json(response, { status: 400 });
		}

		let validatedDisplayName: string | null = null;
		if (displayName?.trim()) {
			const displayNameValidation = isValidDisplayName(displayName);
			if (!displayNameValidation.valid) {
				const response: RegisterResponse = {
					code: 400,
					success: false,
					error: displayNameValidation.error || "Invalid display name",
				};
				return Response.json(response, { status: 400 });
			}
			validatedDisplayName = displayNameValidation.name || null;
		}

		const emailValidation = isValidEmail(email);
		if (!emailValidation.valid) {
			const response: RegisterResponse = {
				code: 400,
				success: false,
				error: emailValidation.error || "Invalid email",
			};
			return Response.json(response, { status: 400 });
		}

		const passwordValidation = isValidPassword(password);
		if (!passwordValidation.valid) {
			const response: RegisterResponse = {
				code: 400,
				success: false,
				error: passwordValidation.error || "Invalid password",
			};
			return Response.json(response, { status: 400 });
		}

		const existingUsernameQuery =
			"SELECT id FROM users WHERE username = ? LIMIT 1";
		const existingUsernameResult = (await cassandra.execute(
			existingUsernameQuery,
			[usernameValidation.username],
		)) as { rows: Array<{ id: string }> };

		if (existingUsernameResult.rows.length > 0) {
			const response: RegisterResponse = {
				code: 409,
				success: false,
				error: "Username already exists",
			};
			return Response.json(response, { status: 409 });
		}

		const existingEmailQuery = "SELECT id FROM users WHERE email = ? LIMIT 1";
		const existingEmailResult = (await cassandra.execute(existingEmailQuery, [
			email.trim().toLowerCase(),
		])) as { rows: Array<{ id: string }> };

		if (existingEmailResult.rows.length > 0) {
			const response: RegisterResponse = {
				code: 409,
				success: false,
				error: "Email already exists",
			};
			return Response.json(response, { status: 409 });
		}

		const userId = pika.gen("user");

		const hashedPassword = await Bun.password.hash(password, {
			algorithm: "argon2id",
			memoryCost: 4096,
			timeCost: 3,
		});

		const now = new Date();
		const insertUserQuery = `
			INSERT INTO users (
				id, username, display_name, email, password,
				is_verified, created_at, updated_at
			) VALUES (?, ?, ?, ?, ?, ?, ?, ?)
		`;

		await cassandra.execute(insertUserQuery, [
			userId,
			usernameValidation.username,
			validatedDisplayName,
			email.trim().toLowerCase(),
			hashedPassword,
			false,
			now,
			now,
		]);

		const responseUser: RegisterResponse["user"] = {
			id: userId,
			username: usernameValidation.username,
			displayName: validatedDisplayName,
			email: email.trim().toLowerCase(),
			isVerified: false,
			createdAt: now.toISOString(),
		};

		const response: RegisterResponse = {
			code: 201,
			success: true,
			message: "User registered successfully",
			user: responseUser,
		};

		try {
			const emailVariables = {
				subject: "Welcome to void",
				companyName: "void",
				id: responseUser.id,
				displayName: responseUser.displayName || responseUser.username,
				isVerified: "Pending verification",
				verificationUrl: `${environment.fqdn}/verify?token=generated_token`, // TODO: Actually generate a token
			};

			mailerService.sendTemplateEmail(
				responseUser.email,
				"Welcome to void",
				"register",
				emailVariables,
			);
		} catch (error) {
			echo.error({
				message: "Failed to send registration email",
				error,
				to: responseUser.email,
				template: "register",
			});
		}

		return Response.json(response, { status: 201 });
	} catch {
		const response: RegisterResponse = {
			code: 500,
			success: false,
			error: "Internal server error",
		};
		return Response.json(response, { status: 500 });
	}
}

export { handler, routeDef };
